@@ -10,6 +10,7 @@ use apexl\Io\modules\company\entities\companyEntity;
 use apexl\Io\modules\display\components\BasicLink;
 use apexl\Io\modules\display\components\ButtonBar;
 use apexl\Io\modules\display\components\CardEntityFilteredDisplayTable;
+use apexl\Io\modules\display\components\ColWrapper;
 use apexl\Io\modules\display\components\ContentTitle;
 use apexl\Io\modules\display\components\DisplayTable;
 use apexl\Io\modules\display\components\EntityFilteredDisplayTable;
@@ -22,6 +23,7 @@ use apexl\Io\modules\formbuilder\includes\checkboxField;
 use apexl\Io\modules\formbuilder\includes\inputField;
 use apexl\Io\modules\formbuilder\includes\markupField;
 use apexl\Io\modules\formbuilder\includes\selectField;
+use apexl\Io\modules\subscription\entities\subscriptionEntity;
 use apexl\Io\modules\subscriptionDisplay\components\viewTiles;
 use apexl\Io\modules\userDisplay\components\dashboardTiles;
 use apexl\Io\modules\userDisplay\components\roleEntityDisplayTile;
@@ -171,6 +173,7 @@ class userController extends Controller
         $currentUser = currentUser::getCurrentUser();
         $form = new formEntity();
         $form->setId('user-manage-settings-form');
+        $form->setSubmitText('Update Now');
         $form->setMethod('post');
         $form->setActionUrl(Routes::getRoutePattern('user.updateSettings'));
         $settings = $currentUser->settings ?? null;
@@ -248,7 +251,7 @@ class userController extends Controller
         $rowWrapper->addComponent(
             (new ContentTitle())
             ->addContent("Please check your email for further instructions. The email may take upto a few minutes to arrive.")
-            ->addClass('text-white')
+            ->addClass('text-white forgot-password-email-message')
                 ->setColWidth('md', 12)->setID('forgottenPassword')
         );
         $rowWrapper->addComponent(
@@ -299,11 +302,16 @@ class userController extends Controller
         }
 
         $form->setActionUrl(Routes::getRoutePattern('userEntity.put'));
+        $form->setSubmitText('Update Now');
 
         if ($this->currentUser->isAllowed('ChangeUserActiveState')) {
-            $activeField = (new checkboxField('active'))->setLabel('Active');
+            $activeField = (new checkboxField('active'))->setLabel('Active (Can Login)');
+            $suspendedField = (new checkboxField('suspended'))->setLabel('Suspended');
+            $invoiceNameField = (new inputField('invoice_name_override'))->setLabel('Invoice Name Override');
         } else {
             $activeField = (new inputField('active'))->setInputType('hidden');
+            $suspendedField = (new inputField('suspended'))->setInputType('hidden');
+            $invoiceNameField = (new inputField('invoice_name_override'))->setInputType('hidden');
         }
 
         if ($id = $args[$user->primaryKey] ?? false) {
@@ -314,9 +322,13 @@ class userController extends Controller
             $form->addField($form->getFormField('email')->setValue($user->email));
 
             $activeField->setValue($user->active);
+            $suspendedField->setValue($user->suspended);
+            $invoiceNameField->setValue($user->invoice_name_override ?? null);
         }
 
         $form->addField($activeField);
+        $form->addField($suspendedField);
+        $form->addField($invoiceNameField);
 
         //allow other modules to add form fields.
         $form = Hook::processHook('displayFormRender', $form, $user, 'userEditForm');
@@ -375,6 +387,9 @@ class userController extends Controller
      */
     public function userList(Request $request, Response $response)
     {
+        $buttonBar = new ButtonBar();
+        $buttonBar->setEntity((new userEntity()), 'List');
+
         $helpComponents = [
             (new ContentTitle())->addClass('text-black text-start help-modal-content-title')->addTitle('A list of Users, and their details'),
             (new DisplayTable())->addClass('text-black text-start help-modal-display-table')->addRows([
@@ -393,14 +408,17 @@ class userController extends Controller
             ->addTitle('Users')
             ->src(Routes::getRoutePattern('userEntity.get'))
             ->displayPagination()
+            ->hideIdColumn()
             ->addProperty('refreshButton', '<i class="fa-light fa-arrows-rotate"></i>')
         //$table->setFilters('orderBy[]=missing_sims&orderBy[]=DESC&orderBy[]=subscription.id&orderBy[]=DESC');
             ->addFilterForm('user.display.forms.filters.userFilter', null, 5, true, 'Filter Users')
             ->addHeaderModal('Users List Table', $helpComponents);
 
+        $table = Hook::processHook('user_list_table_modify', $table);
+
         //render
         $this->render::setPageTitle('Users');
-        $this->output::addResponse($request, $this->render::build($table));
+        $this->output::addResponse($request, $this->render::build([$buttonBar, $table]));
         return System::asJson($response);
     }
 
@@ -441,6 +459,34 @@ class userController extends Controller
         return System::asJson($response);
     }
 
+    public function suspendUser(Request $request, Response $response, $args = [])
+    {
+        $userEntity = new userEntity();
+        $userEntity->load($args['id']);
+        if (!isset($userEntity->id)) {
+            return System::asJson($response, [], 404);
+        }
+
+        $contentTitle = (new ContentTitle())
+            ->addTitle('Are you sure?')
+            ->addContent($userEntity->first_name . ' will no longer be able to log in, and any payment subscriptions they have will be suspended')
+            ;
+        $suspendTile = dashboardTiles::suspendUserTile($userEntity);
+        $components = [$contentTitle, $suspendTile];
+
+        $components = Hook::processHook('suspendUserPopupContent', $components, $userEntity);
+
+        $rowWrapper = (new RowWrapper())
+            ->addComponent((new ColWrapper())
+                ->addComponents($components)
+            );
+
+        $this->render::setActive("modal");
+        $this->output::addResponse($request, $this->render::build($rowWrapper));
+
+        return System::asJson($response);
+    }
+
     /**
      * @param bool $passwordRequired
      * @return formEntity
